require 'sass/switcheroo'
require 'test/unit'

class DSLTest < Test::Unit::TestCase
  FIXTURES_DIR = File.expand_path("fixtures", File.join(File.dirname(__FILE__)))

  def test_basic_dsl
    importer = Sass::Switcheroo.rules(FIXTURES_DIR) do
      replace("**/foo/*", "/foo/" => "/bar/")
    end
    assert_equal Sass::Switcheroo::Importer, importer.class
    file, syntax = importer.find_real_file(FIXTURES_DIR, "replacement/foo/one", {})
    assert_equal File.join(FIXTURES_DIR, *%w(replacement bar one.scss)), file
    assert_equal :scss, syntax
  end

  def test_missing_replacement
    importer = Sass::Switcheroo.rules(FIXTURES_DIR) do
      replace("**/foo/*", "/foo/" => "/bar/")
    end
    assert_equal Sass::Switcheroo::Importer, importer.class
    # missing here 
    file, _syntax = importer.find_real_file(FIXTURES_DIR, "replacement/foo/two", {})
    assert_nil file
  end

  def test_skipped_replacement
    importer = Sass::Switcheroo.rules(FIXTURES_DIR, :fallback => true) do
      replace("**/foo/*", "/foo/" => "/bar/")
    end
    assert_equal Sass::Switcheroo::Importer, importer.class
    file, syntax = importer.find_real_file(FIXTURES_DIR, "replacement/foo/two", {})
    assert_equal File.join(FIXTURES_DIR, *%w(replacement foo two.scss)), file
    assert_equal :scss, syntax
  end

  def test_interacting_replacement
    importer = Sass::Switcheroo.rules(FIXTURES_DIR) do
      replace("**/foo/*", "/foo/" => "/bar/")
      replace("**/one", "/one" => "/one-alternate")
    end
    assert_equal Sass::Switcheroo::Importer, importer.class
    file, syntax = importer.find_real_file(FIXTURES_DIR, "replacement/foo/one", {})
    assert_equal File.join(FIXTURES_DIR, *%w(replacement bar one-alternate.scss)), file
    assert_equal :scss, syntax
  end

  def test_replacement_doesnt_keep_matching
    importer = Sass::Switcheroo.rules(FIXTURES_DIR) do
      # this glob matches itself, shouldn't infinite loop.
      replace("**/one*", "/one" => "/one-alternate")
    end
    assert_equal Sass::Switcheroo::Importer, importer.class
    file, syntax = importer.find_real_file(FIXTURES_DIR, "replacement/bar/one", {})
    assert_equal File.join(FIXTURES_DIR, *%w(replacement bar one-alternate.scss)), file
    assert_equal :scss, syntax
  end

  def test_returns_engine_with_replaced_contents
    importer = Sass::Switcheroo.rules(FIXTURES_DIR) do
      replace("**/foo/*", "/foo/" => "/bar/")
    end
    engine = importer.find("replacement/foo/one", {})
    assert_equal <<CSS, engine.render
.bar .one {
  color: blue; }
CSS
  end

  def test_before_import
    importer = Sass::Switcheroo.rules(FIXTURES_DIR) do
      before("**/foo/*", "around/some_before_file")
    end
    engine = importer.find("replacement/foo/one", {:load_paths => [importer]})
    assert_equal <<CSS, engine.render
.before {
  border: 1px solid green; }

.foo .one {
  color: red; }
CSS
  end

  def test_after_import
    importer = Sass::Switcheroo.rules(FIXTURES_DIR) do
      after("**/foo/*", "around/some_after_file")
    end
    engine = importer.find("replacement/foo/one", {:load_paths => [importer]})
    assert_equal <<CSS, engine.render
.foo .one {
  color: red; }

.after {
  border: 1px solid red; }
CSS
  end

  def test_before_and_after_import
    importer = Sass::Switcheroo.rules(FIXTURES_DIR) do
      before("**/foo/*", "around/some_before_file")
      after("**/foo/*", "around/some_after_file")
    end
    engine = importer.find("replacement/foo/one", {:load_paths => [importer]})
    assert_equal <<CSS, engine.render
.before {
  border: 1px solid green; }

.foo .one {
  color: red; }

.after {
  border: 1px solid red; }
CSS
  end

  def test_before_and_after_patterns
    importer = Sass::Switcheroo.rules(FIXTURES_DIR) do
      before("replacement/**/foo/*", %r{/foo/(.*)} => 'around/foo/before_\1')
      after("replacement/**/foo/*", %r{/foo/(.*)} => 'around/foo/after_\1')
    end

    engine = importer.find("replacement/foo/one", {:load_paths => [importer]})

    assert_equal <<CSS, engine.render
.before .one-foo {
  border-width: 1px; }

.foo .one {
  color: red; }

.after .one-foo {
  border-width: 1px; }
CSS
  end

  def test_before_and_after_patterns_only_when_existing
    importer = Sass::Switcheroo.rules(FIXTURES_DIR) do
      before("replacement/**/foo/*", %r{/foo/(.*)} => 'around/foo/before_\1', :if_exists => true)
      after("replacement/**/foo/*", %r{/foo/(.*)} => 'around/foo/after_\1', :if_exists => true)
    end

    engine = importer.find("replacement/foo/two", {:load_paths => [importer]})

    assert_equal <<CSS, engine.render
.foo .two {
  background-color: red; }
CSS
  end
end
